const path = require("path")
const { Transform } = require("stream")
const babel = require("@babel/core")

const ROOT = path.resolve()
const LEN = ROOT.length


class BabelStream extends Transform {
  constructor (filename, cnf) {
    super()
    if (!(cnf && typeof cnf === "object")) {
      cnf = {}
    }
    cnf.filename = filename || cnf.filename || ROOT
    cnf.sourceFileName = "/script" + cnf.filename.substr(LEN)
    cnf.sourceMaps = cnf.sourceMaps === false ? false : "inline"
    delete cnf._flags
    const opts = babel.loadPartialConfig(cnf).options
    Object.assign(this, {filename, data: [], opts})
  }

  _transform (buffer, enc, callback) {
    this.data.push(buffer)
    callback()
  }

  _flush (callback) {
    const data = Buffer.concat(this.data).toString()
    const self = this

    babel.transform(data, self.opts, (err, result) => {
      if (err) {
        callback(err)
      } else {
        self.push(result !== null ? result.code : data)
        callback()
      }
    })
  }
}

module.exports = function create (filename, options) {
  return new BabelStream(filename, options)
}
